@@ -34,24 +34,19 @@
 #define HBA_VENDOR_SPECIFIC_ID  0
 
 /*
- * Convert fc_port_type values to ascii string name.
- * (This table is copied from scsi_transport_fc.c).
+ * table of /sys port types strings to HBA-API values.
  */
-static struct {
-	enum fc_port_type	value;
-	char			*name;
-} port_types_table[] = {
-    { FC_PORTTYPE_UNKNOWN,     "Unknown" },
-    { FC_PORTTYPE_OTHER,       "Other" },
-    { FC_PORTTYPE_NOTPRESENT,  "Not Present" },
-    { FC_PORTTYPE_NPORT,       "NPort (fabric via point-to-point)" },
-    { FC_PORTTYPE_NLPORT,      "NLPort (fabric via loop)" },
-    { FC_PORTTYPE_LPORT,       "LPort (private loop)" },
-    { FC_PORTTYPE_PTP,         "Point-To-Point (direct nport connection)" },
-    { FC_PORTTYPE_NPIV,        "NPIV VPORT" },
+struct sa_nameval port_types_table[] = {
+	{ "Unknown",		HBA_PORTTYPE_UNKNOWN },
+	{ "Other",		HBA_PORTTYPE_OTHER },
+	{ "Not Present", 	HBA_PORTTYPE_NOTPRESENT },
+	{ "NPort (fabric via point-to-point)",	HBA_PORTTYPE_NPORT },
+	{ "NLPort (fabric via loop)",	HBA_PORTTYPE_NLPORT },
+	{ "LPort (private loop)",	HBA_PORTTYPE_LPORT },
+	{ "Point-To-Point (direct nport connection)", 	HBA_PORTTYPE_PTP },
+	{  "NPIV VPORT",	HBA_PORTTYPE_NPORT },
+	{ NULL, 0 }
 };
-fc_enum_name_search(port_type, fc_port_type, port_types_table)
-#define FC_PORTTYPE_MAX_NAMELEN         50
 
 /*
  * table of /sys port state strings to HBA-API values.
@@ -74,14 +69,56 @@ struct sa_nameval port_states_table[] = {
  * table of /sys port speed strings to HBA-API values.
  */
 struct sa_nameval port_speeds_table[] = {
-	{ "10 Gbit",        HBA_PORTSPEED_10GBIT },
-	{ "2 Gbit",         HBA_PORTSPEED_2GBIT },
-	{ "1 Gbit",         HBA_PORTSPEED_1GBIT },
-	{ "Not Negotiated", HBA_PORTSPEED_NOT_NEGOTIATED },
 	{ "Unknown",        HBA_PORTSPEED_UNKNOWN },
+	{ "1 Gbit",         HBA_PORTSPEED_1GBIT },
+	{ "2 Gbit",         HBA_PORTSPEED_2GBIT },
+	{ "10 Gbit",        HBA_PORTSPEED_10GBIT },
+	{ "Not Negotiated", HBA_PORTSPEED_NOT_NEGOTIATED },
 	{ NULL, 0 }
 };
 
+/*
+ * parse strings from /sys port speed/support_speeds files
+ * and convert them to bitmasks for the HBA_PORTSPEED supported
+ * Format expected: "1 Gbit[, 10 Gbit]", etc.
+ */
+static int sys_read_speed(const char *dir, const char *file, char *buf,
+			  size_t buflen, HBA_PORTSPEED *speeds)
+{
+	int rc = 0;
+	u_int32_t val = 0;
+	int len = 0;
+	char *cp;
+	struct sa_nameval *tp = port_speeds_table;
+
+	rc = sa_sys_read_line(dir, file, buf, buflen);
+	if (rc == 0 && strstr(buf, "Unknown") == NULL) {
+		for (cp = buf; *cp != '\0';) {
+			for (; tp->nv_name != NULL; tp++) {
+				len = strlen(tp->nv_name);
+				if (strncasecmp(tp->nv_name, cp, len) == 0) {
+					val |= tp->nv_val;
+					cp += len;
+					break;
+				}
+			}
+			if (*cp == '\0')
+				break;
+			if (*cp == ',') {
+				cp++;
+				if (*cp == ' ')
+					cp++;
+			}
+			else
+				break; /* invalid string */
+		}
+	}
+
+	*speeds = val;
+
+	return rc;
+}
+
 /*
  * Code for OpenFC-supported adapters.
  */
@@ -152,7 +189,6 @@ sysfs_scan(struct dirent *dp, void *arg)
 	struct adapter_info *ap;
 	struct port_info *pp;
 	char host_dir[80], hba_dir[80], drv_dir[80];
-	struct stat stat_buf;
 	char dev_dir[128];
 	char ifname[20], buf[256];
 	char *driverName;
@@ -302,20 +338,21 @@ sysfs_scan(struct dirent *dp, void *arg)
 
 	/* Get PortType */
 	rc = sa_sys_read_line(pp->host_dir, "port_type", buf, sizeof(buf));
-	pap->PortType = get_fc_port_type_value(buf);
+	rc = sa_enum_encode(port_types_table, buf, &pap->PortType);
 
 	/* Get PortState */
 	rc = sa_sys_read_line(pp->host_dir, "port_state", buf, sizeof(buf));
 	rc = sa_enum_encode(port_states_table, buf, &pap->PortState);
 
 	/* Get PortSpeed */
-	rc = sa_sys_read_line(pp->host_dir, "speed", buf, sizeof(buf));
-	rc = sa_enum_encode(port_speeds_table, buf, &pap->PortSpeed);
+	rc = sys_read_speed(pp->host_dir, "speed",
+				buf, sizeof(buf),
+				&pap->PortSpeed);
 
 	/* Get PortSupportedSpeed */
-	rc = sa_sys_read_line(pp->host_dir, "supported_speed",
-				buf, sizeof(buf));
-	rc = sa_enum_encode(port_speeds_table, buf, &pap->PortSupportedSpeed);
+	rc = sys_read_speed(pp->host_dir, "supported_speeds",
+				buf, sizeof(buf),
+				&pap->PortSupportedSpeed);
 
 	/* Get PortMaxFrameSize */
 	rc = sa_sys_read_line(pp->host_dir, "maxframe_size", buf, sizeof(buf));
@@ -357,7 +394,10 @@ sysfs_scan(struct dirent *dp, void *arg)
 	/* Get PortSupportedClassofService */
 	rc = sa_sys_read_line(pp->host_dir, "supported_classes",
 				buf, sizeof(buf));
-	pap->PortSupportedClassofService = *(strstr(buf, "Class") + 6) - '0';
+
+	cp = strstr(buf, "Class");
+	if (cp)
+		pap->PortSupportedClassofService = *(cp + 6) - '0';
 
 	/* Get OSDeviceName */
 	sa_strncpy_safe(pap->OSDeviceName, sizeof(pap->OSDeviceName),
